#This code analyzes the output of an 100-day simulation set up like the primary 299.5K simulation with 500 air parcels added to it. 
#The analysis is performed on data from the 2-day period from days 50-52. During this period, the code picks out the subset of the trajectories of the 500 parcels that 
#fall within the analysis subdomain (r<=600km).
#Finally, the code plots figure 4 from the main text using variables calculated by this code, as well as output from MAFALDA_parceltests.py

#Import modules
import datetime as dt
from numpy import dtype
import numpy as np
from pylab import *
import netCDF4 as n4
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import gridspec
from math import *
import xarray as xr
from scipy import signal
from scipy import spatial

#Load parcel data
root= '/scratch/groups/oneillm/Laurel/SST_exps_3D/axisymm_exps/Parcel_tests/2-day-period/' 
ds=xr.open_dataset(root + 'cm1out_pdata.nc')
ds

#Load variables
rho=ds['rho'] #Dry-air density
w=ds['w'] 
th=ds['th'] #Dry potential temperature
qv=ds['qv']
v=ds['v']
qc=ds['qc']
qi=ds['qi']
qs=ds['qs']
qg=ds['qg']
z=ds['z'] #Height in m
qr=ds['qr']
x=ds['x'] #Radial position. 
time=ds['time'] #Number of seconds since simulation began. 
p=ds['prs'] #Pressure
t=ds['t']
idnum=ds['xh'] #Parcel ID number

rmax=600
tstart = 0 #Start time (number of timesteps since the first output timestep on day 50)
tgap = 1 
tend = 24 #End time

#Define constants

R = 287 #Gas constant for dry air (J/(K kg)). 
Rv = 461.5 #Water vapor gas constant (J/(K kg)). 
T_f=273.15 #Freezing temperature (K)
epsil=R/Rv
p_ref=100000 #Reference pressure in Pa. 
T_ref=T_f #Reference temperature in K. 
ee=0.01*p*qv/(epsil+qv) #Same as ep but in hPa. Matches CM1 formulation.
tlcl=55.0+2840.0/(3.5*np.log(t)-np.log(1.0e-20+ee)-4.805) #Temperature at the LCL. Matches CM1 formulation.
th_e=t*((p_ref/p)**(0.2854*(1.0-0.28*qv)))*np.exp(((3376.0/tlcl)-2.54)*qv*(1.0+0.81*qv)) #Equivalent potential temperature. Matches CM1 formulation

#Pick parts of parcel paths that exist in the analysis subdomain during this time period
idbox=[] #ID numbers of parcels that fulfill this criterion for all or part of the 2-day period
indices_idbox=[] #Time indices corresponding to when a given parcel with ID number in idbox was in the analysis subdomain
for idnum in range(0, 500): #Start with this, then do all 500 parcels!
    indices=np.where((x[tstart:tend:tgap,idnum]/1000<rmax))
    if np.size(indices)>0: #If array is not empty
        idbox.append(idnum)
        indices_idbox.append(indices)

#Make Figure 4!

#Load the output of MAFALDA_parceltests.py (i.e., the deepest trajectory from the isentropic streamfunction) 
interpolant_z_arr=np.load("Interpolated_trajs.npz", allow_pickle=True)["interpolant_z_arr"]
interpolant_th_arr=np.load("Interpolated_trajs.npz", allow_pickle=True)["interpolant_th_arr"]



plt.rc('figure', titlesize=20)     # fontsize of the figure title
plt.rc('axes', labelsize=15)    # fontsize of the x and y labels
plt.rc('xtick', labelsize=15)    # fontsize of the x tick labels
plt.rc('ytick', labelsize=15)    # fontsize of the x tick labels
plt.rc('legend', fontsize=15)            # fontsize of the legend text

fig, ax=plt.subplots(1, 2, figsize=(15, 8))

#Plot r,z position for all 500 parcels during simulation

for idnum in range (0,500): #plots parcels with IDs 20 to 100   
    ax[0].plot(x[tstart:tend:tgap,idnum]/1000,z[tstart:tend:tgap,idnum]/1000, label = 'ID = %s' %idnum) #Plots radius and z position in kilometers
    ax[0].set_xlabel('Radial Position (km)')
    ax[0].set_ylabel('Height (km)')
    ax[0].set_title('Parcel Positions', fontsize=20)

ax[0].vlines(x=rmax, ymin=0, ymax=16, color='k', linewidth=2) #Vertical line indicating radial extent of analysis subdomain
ax[0].set_ylim(-0.5, 16)
ax[0].grid()

#Plot th_e,z positions of the parcel paths within the analysis subdomain only, as well as the deepest MAFALDA trajectory.

#Plot deepest MAFALDA trajectory
ax[1].plot(interpolant_th_arr[-1],interpolant_z_arr[-1]/1000, color='black', linewidth=2, label='Outermost MAFALDA trajectory')

#Plot parcel paths
for i in np.arange(len(idbox)):
    idnum=idbox[i]
    if len(indices_idbox[i][0])>1: #If more than one point exists for the current parcel's path that lies inside the analysis subdomain
        ax[1].plot(th_e[indices_idbox[i][0],idnum],z[indices_idbox[i][0],idnum]/1000) #Plot z position in kilometers
    ax[1].set_xlabel('Equivalent Potential Temperature (K)')
    ax[1].set_ylabel('Height (km)')
    ax[1].set_title('MAFALDA and Parcels Comparison', fontsize=20)

fig.patch.set_facecolor('white')

ax[1].set_ylim(-0.5, 16)
ax[1].grid() 
ax[1].legend(fontsize=15)

plt.tight_layout()

plt.savefig('Figure4.png', facecolor='white')

